==> elements-0.0.5.tar.gz: lib/src/StopwatchElement.dart <==
import 'dart:async';
import 'dart:html';
import 'dart:math';
import 'Elements_base.dart';

final double HALF_PI = 0.5 * PI;
final double DOUBLE_PI = 2 * PI;

// An analog stopwatch
// param int duration: timeout duration in milliseconds, must be dividable by 4
// param DivElement container: Html container which will hold the stopwatch
class StopwatchElement extends afElement 
{
  StopwatchElement(this._duration, this._container, { Duration delay, Function onDone : null })
  {
    if (_duration == null) throw new ArgumentError.notNull("duration");
    if (_container == null) throw new ArgumentError.notNull("container");
    if (_duration <= 0) throw new ArgumentError("Duration must be > 0 seconds)");
    
    _callback = onDone;
    
    _quarterDurationSeconds = _duration.toDouble()~/4000;   
    _firstFrame = true;
    
    _radiansPerMillisecond = DOUBLE_PI / _duration;
    
    String widthStr = _container.getComputedStyle().width;
    _dimension = int.parse(widthStr.substring(0, widthStr.indexOf("px")));
    _halfDimension = 0.5 * _dimension;
    _margin = _dimension / 7;
    
    _container.style.position = "relative";
    _container.style.margin = (0.5*_margin).toString() + "px";
    _q0 = new SpanElement();
    _q1 = new SpanElement();
    _q2 = new SpanElement();
    _q3 = new SpanElement();
    _q0.style.color = _q1.style.color = _q2.style.color = _q3.style.color = "#000";
    _q0.style.fontWeight = _q1.style.fontWeight = _q2.style.fontWeight = _q3.style.fontWeight = "bold";
    _q0.style.fontSize = _q1.style.fontSize = _q2.style.fontSize = _q3.style.fontSize = "18px";
    _q0.style.position = _q1.style.position = _q2.style.position = _q3.style.position = "absolute";
    _q0.style.bottom = _q2.style.top = (_dimension - 0.5*_margin).toString() + "px";
    _q0.style.left = _q2.style.left = "0";
    _q0.style.textAlign = _q2.style.textAlign = "center";
    _q0.style.width = _q2.style.width = "100%"; 
    _q1.style.left = _q3.style.right = (_dimension - 0.5*_margin).toString() + "px";
    _q1.style.top = _q3.style.top = (_halfDimension - 9).toString() + "px";    
    
    _q0.innerHtml = "0";
    _q1.innerHtml = _quarterDurationSeconds.toString();
    _q2.innerHtml = (2*_quarterDurationSeconds).toString();
    _q3.innerHtml = (3*_quarterDurationSeconds).toString();
    
    _canvas = new CanvasElement(width:_dimension, height:_dimension);
    _context = _canvas.getContext("2d");
    _context.lineWidth = 4;

    _container.children.add(_canvas);
    _container.children.add(_q0);
    _container.children.add(_q1);
    _container.children.add(_q2);
    _container.children.add(_q3);
    
    _currentRadians = -DOUBLE_PI;
    
    if (delay != null) new Timer(delay, () => _render(null));
    else _render(null);
  }
  
  void _render(double num)
  {
    if (num != null)
    {
      if (_firstFrame == true)
      {
        _animationStartTime = num;
        _firstFrame = false;
      }
      
      _currentRadians = -HALF_PI + (num - _animationStartTime) * _radiansPerMillisecond;
      _context.clearRect(0, 0, _dimension, _dimension);
      _context.setFillColorRgb(0, 173, 217);
      _context.setStrokeColorRgb(0, 203, 247);
  
      _context.beginPath();
      _context.arc(_halfDimension, _halfDimension, _halfDimension - _margin, 0, DOUBLE_PI);
      _context.fill();
      _context.stroke();
      _context.beginPath();
      _context.moveTo(_halfDimension, _halfDimension);
      _context.setFillColorRgb(255, 255, 255);
      _context.arc(_halfDimension, _halfDimension, _halfDimension - _margin - 0.5*_context.lineWidth, -HALF_PI, _currentRadians);
      _context.fill();
      _context.stroke();
    }
    
    if (_currentRadians < DOUBLE_PI - HALF_PI) window.requestAnimationFrame(_render);
    else if (_callback != null) _callback();
  }
  
  
  bool _firstFrame;
  int _dimension, _duration, _quarterDurationSeconds;
  var _context;  
  double _radiansPerMillisecond, _currentRadians, _halfDimension, _margin, _animationStartTime;
  DivElement _container;
  SpanElement _q0, _q1, _q2, _q3;
  CanvasElement _canvas;
  Function _callback;
}

==> elements-0.0.6.tar.gz: lib/src/StopwatchElement.dart <==
import 'dart:async';
import 'dart:html';
import 'dart:math';
import 'Elements_base.dart';

final double HALF_PI = 0.5 * PI;
final double DOUBLE_PI = 2 * PI;

// An analog stopwatch
// param int duration: timeout duration in milliseconds, must be dividable by 4
// param DivElement container: Html container which will hold the stopwatch
class StopwatchElement extends afElement 
{
  StopwatchElement(this._duration, this._container, { Duration delay, Function onDone : null })
  {
    if (_duration == null) throw new ArgumentError.notNull("duration");
    if (_container == null) throw new ArgumentError.notNull("container");
    if (_duration <= 0) throw new ArgumentError("Duration must be > 0 seconds)");
    
    _stopped = false;
    
    _callback = onDone;
    
    _quarterDurationSeconds = _duration.toDouble()~/4000;   
    _firstFrame = true;
    
    _radiansPerMillisecond = DOUBLE_PI / _duration;
    
    String widthStr = _container.getComputedStyle().width;
    _dimension = int.parse(widthStr.substring(0, widthStr.indexOf("px")));
    _halfDimension = 0.5 * _dimension;
    _margin = _dimension / 7;
    
    _container.style.position = "relative";
    _container.style.margin = (0.5*_margin).toString() + "px";
    _q0 = new SpanElement();
    _q1 = new SpanElement();
    _q2 = new SpanElement();
    _q3 = new SpanElement();
    _q0.style.color = _q1.style.color = _q2.style.color = _q3.style.color = "#000";
    _q0.style.fontWeight = _q1.style.fontWeight = _q2.style.fontWeight = _q3.style.fontWeight = "bold";
    _q0.style.fontSize = _q1.style.fontSize = _q2.style.fontSize = _q3.style.fontSize = "18px";
    _q0.style.position = _q1.style.position = _q2.style.position = _q3.style.position = "absolute";
    _q0.style.bottom = _q2.style.top = (_dimension - 0.5*_margin).toString() + "px";
    _q0.style.left = _q2.style.left = "0";
    _q0.style.textAlign = _q2.style.textAlign = "center";
    _q0.style.width = _q2.style.width = "100%"; 
    _q1.style.left = _q3.style.right = (_dimension - 0.5*_margin).toString() + "px";
    _q1.style.top = _q3.style.top = (_halfDimension - 9).toString() + "px";    
    
    _q0.innerHtml = "0";
    _q1.innerHtml = _quarterDurationSeconds.toString();
    _q2.innerHtml = (2*_quarterDurationSeconds).toString();
    _q3.innerHtml = (3*_quarterDurationSeconds).toString();
    
    _canvas = new CanvasElement(width:_dimension, height:_dimension);
    _context = _canvas.getContext("2d");
    _context.lineWidth = 4;

    _container.children.add(_canvas);
    _container.children.add(_q0);
    _container.children.add(_q1);
    _container.children.add(_q2);
    _container.children.add(_q3);
    
    _currentRadians = -DOUBLE_PI;
    
    if (delay != null) new Timer(delay, () => _render(null));
    else _render(null);
  }
  
  void stop()
  {
    _stopped = true;
  }
  
  void _render(double num)
  {
    if (num != null)
    {
      if (_firstFrame == true)
      {
        _animationStartTime = num;
        _firstFrame = false;
      }
      
      _currentRadians = -HALF_PI + (num - _animationStartTime) * _radiansPerMillisecond;
      _context.clearRect(0, 0, _dimension, _dimension);
      _context.setFillColorRgb(0, 173, 217);
      _context.setStrokeColorRgb(0, 203, 247);
  
      _context.beginPath();
      _context.arc(_halfDimension, _halfDimension, _halfDimension - _margin, 0, DOUBLE_PI);
      _context.fill();
      _context.stroke();
      _context.beginPath();
      _context.moveTo(_halfDimension, _halfDimension);
      _context.setFillColorRgb(255, 255, 255);
      _context.arc(_halfDimension, _halfDimension, _halfDimension - _margin - 0.5*_context.lineWidth, -HALF_PI, _currentRadians);
      _context.fill();
      _context.stroke();
    }
    
    if (_currentRadians < DOUBLE_PI - HALF_PI && !_stopped) window.requestAnimationFrame(_render);
    else if (_callback != null) _callback();
  }
  
  bool _stopped;
  bool _firstFrame;
  int _dimension, _duration, _quarterDurationSeconds;
  var _context;  
  double _radiansPerMillisecond, _currentRadians, _halfDimension, _margin, _animationStartTime;
  DivElement _container;
  SpanElement _q0, _q1, _q2, _q3;
  CanvasElement _canvas;
  Function _callback;
}
